<?php
namespace Controller;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Util\Messages;
use Util\Utilities;

class TaskFeatures extends \Generated\Controller\TaskFeatures
{

    public function __construct(Manager $dataSourceManager, Utilities $utilities)
    {
        $this->utilities = $utilities;
        parent::__construct($dataSourceManager);
    }

    /**
     * Action used for insert new rows and update the old ones.
     *
     * @param DTO\Request\DataSet $request  A DTO\Request\DataSet with all rows to be persisted..
     * @param DTO\Response        $response A DTO\Response, mostly of cases will only add messages.
     *
     * @return void
     */

    public function save(DTO\Request $request, DTO\Response $response, $showMessage = false)
    {
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
        $dataSet = new \Zeedhi\Framework\DataSource\DataSet($dataSet->getDataSourceName(),$rows);
        $request = new DTO\Request\DataSet($dataSet,$request->getMethod(),$request->getRoutePath(),$request->getUserId());
        parent::save($request, $response, false);
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response, $showMessage = false)
    {
        parent::find($request, $response, false);
        $dataSets = $response->getDataSets();
        if(!empty($dataSets)){
            $dataSet = $dataSets[0];
            $rows = $dataSet->getRows();
            foreach ($rows as &$row) {
                $row['TIME_BASE_SEG'] = $row['TIME_BASE'];
            }
            $dataSet = new \Zeedhi\Framework\DataSource\DataSet($dataSet->getDataSourceName(),$rows);
            $response->setDataSets([$dataSet]);
        }
    }

    public function delete(DTO\Request $request, DTO\Response $response, $showMessage = false)
    {
        try {
            parent::delete($request, $response, false);
        } catch (\Exception $e) {
            $isUsed = strpos($e->getMessage(), "ORA-02292");
            $response->setStatus(Response::STATUS_ERROR);
            $message = $e->getMessage();
            if ($isUsed) {
                $message = Messages::ERR_TASK_FEATURE_ALREADY_USED;
            }
            throw new \Exception($message);
        }
    }
}