<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 15/10/2015
 * Time: 09:43
 */

namespace Service;


use Doctrine\ORM\EntityManager;
use Util\Utilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Util\InstanceProvider;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class CalendarDetail
{
    const RECURRENCE_TYPE_DAILY = 'D';
    const RECURRENCE_TYPE_WEEKLY = 'W';
    const RECURRENCE_TYPE_MONTHLY = 'M';
    const RECURRENCE_TYPE_YEARLY = 'Y';

    const WEEK_TRUE = 'Y';
    const WEEK_FALSE = 'N';

    const EXCEPTION_TRUE = 'Y';
    const EXCEPTION_FALSE = 'N';

    const WEEK_DAY_DEFAULT = 'W';
    const WEEK_DAY_NOT_DEFAULT = 'O';

    const OUT_DAY_TYPE = 'O';
    const WORK_TYPE = 'W';
    /**
     * @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\CalendarDetail
     */
    public $calendarDetailRepository;
    /**
     * @var EntityManager
     */
    private $entityManager;
    /**
     * @var \Factory\CalendarDetail
     */
    public $calendarDetailFactory;
    /**
     * @var Utilities
     */
    private $environment;
    /**
     * @var \Factory\CalendarWeeklyDays
     */
    private $calendarWeeklyDaysFactory;
    /**
     * @var CalendarDates
     */
    private $calendarDatesService;
    /**
     * @var CalendarWeeklyDays
     */
    private $calendarWeeklyDaysService;

    private $calendarRepository;

    public function __construct(EntityManager $entityManager, Utilities $environment,
                                \Factory\CalendarDetail $calendarDetailFactory,CalendarWeeklyDays $calendarWeeklyDaysService,
                                \Factory\CalendarWeeklyDays $calendarWeeklyDaysFactory,CalendarDates $calendarDatesService)
    {

        $this->entityManager = $entityManager;
        $this->calendarDetailFactory = $calendarDetailFactory;
        $this->environment = $environment;
        $this->calendarDetailRepository = $entityManager
            ->getRepository(Utilities::REPOSITORY_DIR . 'CalendarDetail');
        $this->calendarRepository = $entityManager->getRepository(Utilities::REPOSITORY_DIR . 'Calendar');
        $this->calendarWeeklyDaysFactory = $calendarWeeklyDaysFactory;
        $this->calendarDatesService = $calendarDatesService;
        $this->calendarWeeklyDaysService = $calendarWeeklyDaysService;
    }
    
    public function beforeSaveException($id, $recurrenceType, $name, $calendarId, $type, $status, \DateTime $beginDate, $occurrences,
                                  $exceptionQuantitative, $exceptionDataEnd, $endDate, $every, $sun, $mon, $tue, $wed,
                                  $thu, $fri, $sat, $yearDayException, $monthlyDay, $monthly, $weekDayException,
                                  $orderDay, $weekDay, $yearDay, $workTime, $isApplyChildrenProject = false, $projectId = null)
    {
        $calendarDetail = $this->saveException($id, $recurrenceType, $name, $calendarId, $type, $status, $beginDate, $occurrences,
                                  $exceptionQuantitative, $exceptionDataEnd, $endDate, $every, $sun, $mon, $tue, $wed,
                                  $thu, $fri, $sat, $yearDayException, $monthlyDay, $monthly, $weekDayException,
                                  $orderDay, $weekDay, $yearDay, $workTime);
                                  
        if($isApplyChildrenProject && !$id){
            $projectService     = InstanceProvider::getService()->getProject();
            $projectRepository  = InstanceProvider::getRepository()->getProject();
            $calendarRepository = InstanceProvider::getRepository()->getCalendar();
            /** @var Entities\Project[] $childrenProjects */
            $childrenProjects = $projectRepository->getProjectChildrenNotCanceledAndNotConcludedById($projectId);
            
            if(!empty($childrenProjects)){
                $calendarParent = $calendarRepository->find($calendarId);
                $calendarDefaultId = $calendarParent->getCalendarDefault()->getId();

                foreach($childrenProjects as $projectChildren){

                    if(!$projectChildren->getCalendar()){
                        $calendar = $projectService->saveEditCalendarChildrenProject($projectChildren, $calendarDefaultId);
                    } else {
                        $calendar = $calendarRepository->find($projectChildren->getCalendar());
                    }
                    
                    $this->saveException($id, $recurrenceType, $name, $calendar->getId(), $type, $status, $beginDate, $occurrences,
                                  $exceptionQuantitative, $exceptionDataEnd, $endDate, $every, $sun, $mon, $tue, $wed,
                                  $thu, $fri, $sat, $yearDayException, $monthlyDay, $monthly, $weekDayException,
                                  $orderDay, $weekDay, $yearDay, $workTime);
                }
            }
        }
        
        return $calendarDetail;
    }

    public function saveException($id, $recurrenceType, $name, $calendar, $type, $status, \DateTime $beginDate, $occurrences,
                                  $exceptionQuantitative, $exceptionDataEnd, $endDate, $every, $sun, $mon, $tue, $wed,
                                  $thu, $fri, $sat, $yearDayException, $monthlyDay, $monthly, $weekDayException,
                                  $orderDay, $weekDay, $yearDay, $workTime)
    {

        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            //insert or update
            $calendar = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Calendar', $calendar);
            list($endDate, $occurrences) = $this->setEndDateOrOccurrences($recurrenceType, $beginDate, $occurrences,
                $exceptionQuantitative, $endDate, $every, $sun, $mon, $tue, $wed, $thu, $fri, $sat, $yearDayException,
                $monthlyDay, $monthly, $orderDay, $weekDay, $yearDay);
            if ($id) {
                /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\CalendarDetail $calendarDetail */
                $calendarDetail = $this->calendarDetailRepository->find($id);
                $calendarDetail = $this->calendarDetailFactory->updateExceptionBase($calendarDetail, $calendar, $name,
                    $status, $beginDate, $endDate, $occurrences, $recurrenceType);
            } else {
                $calendarDetail = $this->calendarDetailFactory->createExceptionBase($calendar, $name, $status, $beginDate,
                    $endDate, $occurrences, $recurrenceType);
            }
            list($calendarDetail, $workTime) = $this->setWorkTimeByType($type, $workTime, $calendarDetail);
            $calendarDetail = $this->setCalendarDetailByRecurrence($recurrenceType, $status, $every, $sun, $mon, $tue, $wed,
                $thu, $fri, $sat, $yearDayException, $monthlyDay, $monthly, $orderDay, $weekDay, $yearDay, $workTime,
                $calendarDetail);
            $this->calendarDatesService->setDatesByCalendarDetail($calendarDetail);
            $this->entityManager->persist($calendarDetail);
            $this->entityManager->flush($calendarDetail);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        return $calendarDetail;
    }


    /**
     * @param $calendarDetail
     * @param bool $flush
     * @throws \Exception
     */
    public function removeAllWeeklyDays($calendarDetail, $flush = false )
    {
        try {
            $this->calendarWeeklyDaysService->removeAllWeeklyDays($calendarDetail, true);

            if ($flush) {
                $this->entityManager->flush();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function removeCalendarDetail($calendarDetailId)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $calendarDetail  = $this->calendarDetailRepository->find($calendarDetailId);
            $calendarDetailLinks = $this->calendarDetailRepository->findBy(array('originCalendarDetail' => $calendarDetailId));
            foreach($calendarDetailLinks as $calendarDetailLink){
                $this->removeAllWeeklyDays($calendarDetailLink,true);
                $this->entityManager->remove($calendarDetailLink);
                $this->entityManager->flush();
            }
            $this->removeAllWeeklyDays($calendarDetail,true,true);
            $this->entityManager->remove($calendarDetail);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }

    }

    public function saveCalendarWeeklyDay($calendarDetail, $numberWeek, $workTime, $status, $type = self::WEEK_DAY_NOT_DEFAULT)
    {
        $calendarWeeklyDay = $this->calendarWeeklyDaysFactory->createCalendarWeeklyDay($calendarDetail, $numberWeek,
            $workTime, $type, $status);
        $this->entityManager->persist($calendarWeeklyDay);
        return $calendarWeeklyDay;
    }

    /**
     * @param $status
     * @param $every
     * @param $sun
     * @param $mon
     * @param $tue
     * @param $wed
     * @param $thu
     * @param $fri
     * @param $sat
     * @param $workTime
     * @param $calendarDetail
     * @return \Zeedhi\DevOps\Model\Projectmanagement\Entities\CalendarDetail
     */
    protected function setRecurrenceWeekly($status, $every, $sun, $mon, $tue, $wed, $thu, $fri, $sat, $workTime,
                                           $calendarDetail)
    {
        $calendarDetail = $this->calendarDetailFactory->setRecurrenceWeekly($calendarDetail, $every);
        $weeks = array(null, $sun, $mon, $tue, $wed, $thu, $fri, $sat);
        foreach ($weeks as $numberWeek => $week) {
            if ($week == self::WEEK_TRUE) {
                $this->saveCalendarWeeklyDay($calendarDetail, $numberWeek, $workTime, $status,
                    self::WEEK_DAY_NOT_DEFAULT);
            }
        }
        return $calendarDetail;
    }

    /**
     * @param $every
     * @param $yearDayException
     * @param $monthlyDay
     * @param $orderDay
     * @param $weekDay
     * @param $calendarDetail
     * @return \Zeedhi\DevOps\Model\Projectmanagement\Entities\CalendarDetail
     */
    protected function setRecurrenceMonthly($every, $yearDayException, $monthlyDay, $orderDay, $weekDay, $calendarDetail)
    {
        if (self::EXCEPTION_TRUE == $yearDayException) {
            $calendarDetail = $this->calendarDetailFactory->setRecurrenceMonthlyDay($calendarDetail, $every,
                $monthlyDay);
        } else {
            $calendarDetail = $this->calendarDetailFactory->setRecurrenceMonthlyWeek($calendarDetail, $every,
                $weekDay, $orderDay);
        }
        return $calendarDetail;
    }

    public function findCalendarException($calendarId,$id = null, $filter = '')
    {
        $organization = $this->environment->getCurrentOrganizationId();
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Calendar $calendar */
        $calendar = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Calendar',$calendarId);
        $calendarExceptions = $this->calendarDetailRepository->getCalendarException($calendarId,$organization,$id, $filter);
        foreach($calendarExceptions as &$calendarException){
            if($calendarException['ORIGIN_CALENDAR_DETAIL']){
                $calendarException['TO_CALENDAR'] = 'Não';
                $calendarException['EDIT'] = false;
            }else{
                $calendarException['TO_CALENDAR'] = 'Sim';
                $calendarException['EDIT'] = true;
            }
        }
        if($calendar && $calendar->getCalendarDefault()){
            $defaultCalendarExceptions = $this->calendarDetailRepository->getCalendarException(
                $calendar->getCalendarDefault()->getId(),$organization,$id, $filter);
            foreach($defaultCalendarExceptions as &$defaultCalendarException){
                $defaultCalendarException['TO_CALENDAR'] = 'Não';
                $defaultCalendarException['EDIT'] = false;
            }
            $calendarExceptions = array_merge($calendarExceptions,$defaultCalendarExceptions);
        }
        return $calendarExceptions;
    }
    public function findCalendarExceptionByStructure($structureId,$id = null){
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Structure $structure */
        $structure = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'Structure')->find($structureId);
        $calendar = Utilities::getNextCalendar($structure);
        if($calendar){
            $calendarId = $calendar->getId();
        }else{
            $calendarId = 0;
        }
        $calendarExceptions = $this->findCalendarException($calendarId,$id);
        if(!$structure->getCalendar() || $calendar->getId() != $structure->getCalendar()->getId()){
            foreach($calendarExceptions as &$calendarException){
                $calendarException['TO_CALENDAR'] = 'Não';
                $calendarException['EDIT'] = false;
            }
        }
        return $calendarExceptions;
    }
    public function findCalendarExceptionByResource(Entities\Resource $resource, Entities\Structure $structure = null,$id = null){
        $calendarExceptions = array();
        $structureCalendar = null;
        if($structure){
            $structureCalendar = Utilities::getNextCalendar($structure);
        }
        if($structureCalendar) {
            $calendarExceptions = $this->findCalendarException($structureCalendar->getId(), $id);
            foreach ($calendarExceptions as &$calendarException) {
                $calendarException['TO_CALENDAR'] = 'Não';
                $calendarException['EDIT'] = false;
            }
        }
        if($resource->getCalendar()){
            $calendarExceptions = array_merge($this->findCalendarException($resource->getCalendar()->getId(), $id),$calendarExceptions);
            $calendarExceptions = Utilities::getUniqueArrayMultidimensional($calendarExceptions);
        }
        return $calendarExceptions;
    }
    public function findCalendarExceptionByProject(Entities\Project $project, Entities\Structure $structure,$id = null, $filter = ''){
        $calendarExceptions = array();
        $structureCalendar = Utilities::getNextCalendar($structure);
        if($structureCalendar) {
            $calendarExceptions = $this->findCalendarException($structureCalendar->getId(), $id, $filter);
            foreach ($calendarExceptions as &$calendarException) {
                $calendarException['TO_CALENDAR'] = 'Não';
                $calendarException['EDIT'] = false;
            }
        }
        if($project->getCalendar()){
            $calendarExceptions = array_merge($this->findCalendarException($project->getCalendar()->getId(), $id, $filter),$calendarExceptions);
            $calendarExceptions = Utilities::getUniqueArrayMultidimensional($calendarExceptions);
        }
        return $calendarExceptions;
    }
    public function findCalendarExceptionByProjectResource(Entities\ResourceProject $projectResource,$id = null)
    {
        $project = $projectResource->getProject();
        $resource = $projectResource->getResourceC();
        $calendarExceptions = $this->findCalendarExceptionByProject($project, $project->getStructure(), $id);
        $structure = $this->environment->getStructureForResource($resource);
        $calendarExceptions = array_merge($this->findCalendarExceptionByResource($resource, $structure, $id), $calendarExceptions);
        foreach ($calendarExceptions as &$calendarException) {
            $calendarException['TO_CALENDAR'] = 'Não';
            $calendarException['EDIT'] = false;
        }
        if ($projectResource->getCalendar()) {
            $calendarExceptions = array_merge($this->findCalendarException($projectResource->getCalendar()->getId(), $id), $calendarExceptions);
        }
        $calendarExceptions = Utilities::getUniqueArrayMultidimensional($calendarExceptions);
        return $calendarExceptions;
    }

    public function getOccurrencesForWeekRecurrence(\DateTime $beginDate, \DateTime $endDate, $every, $sun, $mon, $tue,
                                                    $wed, $thu, $fri, $sat)
    {
        $weeks = array(null, $sun, $mon, $tue, $wed, $thu, $fri, $sat);
        $weekdays = array();
        foreach ($weeks as $numberWeek => $week) {
            if ($week == self::WEEK_TRUE) {
                array_push($weekdays, $numberWeek);
            }
        }
        $weekdaysString = implode($weekdays, ', ');
        $occurrences = $this->calendarDetailRepository
            ->getOccurrencesForWeekRecurrence($beginDate->format(Utilities::FORMAT_DATE_DEFAULT),
                $endDate->format(Utilities::FORMAT_DATE_DEFAULT), $weekdaysString, $every);
        return $occurrences;
    }

    public function getEndDateForWeekRecurrence(\DateTime $beginDate, $occurrences, $every, $sun, $mon, $tue, $wed,
                                                $thu, $fri, $sat)
    {
        $weeks = array(null, $sun, $mon, $tue, $wed, $thu, $fri, $sat);
        $weekdays = array();
        foreach ($weeks as $numberWeek => $week) {
            if ($week == self::WEEK_TRUE) {
                array_push($weekdays, $numberWeek);
            }
        }
        $weekdaysString = implode(', ', $weekdays);
        $weekArray = $this->calendarDetailRepository
            ->getEndDateForWeekRecurrence($beginDate->format(Utilities::FORMAT_DATE_DEFAULT),
                $occurrences, $weekdaysString, $every);
        $firstWeekYear = $this->calendarDetailRepository
            ->getDate($beginDate->format(Utilities::FORMAT_DATE_DEFAULT))['WEEK_IN_YEAR'];
        $firstWeekYear -= $every;
        $occurrencesCount = 0;
        $endDate = null;
        $pass = true;
        foreach ($weekArray as $key => $week) {
            if ((($week['WEEK_IN_YEAR'] - $firstWeekYear) % $every == 0) && $occurrencesCount < $occurrences) {
                $occurrencesCount++;
            }
            if ($occurrencesCount == $occurrences && $pass) {
                $endDate = $week['DATA'];
                $pass = false;
            }
        }
        if (empty($endDate)) {
            throw new \Exception('Dia da semana invalida');
        }
        return \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT_BY_BD, $endDate);
    }

    public function getOccurrencesForMonthlyRecurrence(\DateTime $beginDate, \DateTime $endDate, $every,
                                                       $yearDayException, $monthlyDay, $orderDay, $weekDay)
    {
        if (self::EXCEPTION_TRUE == $yearDayException) {
            $occurrences = $this->calendarDetailRepository
                ->getOccurrencesForMonthlyDayRecurrence($beginDate->format(Utilities::FORMAT_DATE_DEFAULT),
                    $endDate->format(Utilities::FORMAT_DATE_DEFAULT), $monthlyDay, $every, $beginDate->format('m'));
        } else {
            $occurrences = $this->calendarDetailRepository
                ->getOccurrencesForMonthlyWeekRecurrence($beginDate->format(Utilities::FORMAT_DATE_DEFAULT),
                    $endDate->format(Utilities::FORMAT_DATE_DEFAULT), $orderDay, $weekDay, $every,
                    $beginDate->format('m'));
        }
        return $occurrences;
    }

    public function getEndDateForMonthlyRecurrence(\DateTime $beginDate, $occurrences, $every,
                                                   $yearDayException, $monthlyDay, $orderDay, $weekDay)
    {
        if (self::EXCEPTION_TRUE == $yearDayException) {
            $occurrences = $this->calendarDetailRepository
                ->getEndDateForMonthlyDayRecurrence($beginDate->format(Utilities::FORMAT_DATE_DEFAULT),
                    $occurrences, $monthlyDay, $every, $beginDate->format('m'));
        } else {
            $occurrences = $this->calendarDetailRepository
                ->getEndDateForMonthlyWeekRecurrence($beginDate->format(Utilities::FORMAT_DATE_DEFAULT),
                    $occurrences, $orderDay, $weekDay, $every, $beginDate->format('m'));
        }
        return $occurrences;
    }

    public function getOccurrencesForYearlyRecurrence(\DateTime $beginDate, \DateTime $endDate, $every,
                                                      $yearDayException, $monthly, $orderDay, $weekDay, $yearDay)
    {
        if (self::EXCEPTION_TRUE == $yearDayException) {
            $occurrences = $this->calendarDetailRepository
                ->getOccurrencesForYearlyDayRecurrence($beginDate->format(Utilities::FORMAT_DATE_DEFAULT),
                    $endDate->format(Utilities::FORMAT_DATE_DEFAULT), $yearDay[0], $yearDay[1]);
        } else {
            $occurrences = $this->calendarDetailRepository
                ->getOccurrencesForYearlyWeekRecurrence($beginDate->format(Utilities::FORMAT_DATE_DEFAULT),
                    $endDate->format(Utilities::FORMAT_DATE_DEFAULT), $orderDay, $weekDay, $monthly);
        }
        return $occurrences;
    }

    public function getEndDateForYearlyRecurrence(\DateTime $beginDate, $occurrences, $every,
                                                  $yearDayException, $monthly, $orderDay, $weekDay, $yearDay)
    {
        if (self::EXCEPTION_TRUE == $yearDayException) {
            $occurrences = $this->calendarDetailRepository
                ->getEndDateForYearlyDayRecurrence($beginDate->format(Utilities::FORMAT_DATE_DEFAULT),
                    $occurrences, $yearDay[0], $yearDay[1]);
        } else {
            $occurrences = $this->calendarDetailRepository
                ->getEndDateForYearlyWeekRecurrence($beginDate->format(Utilities::FORMAT_DATE_DEFAULT),
                    $occurrences, $orderDay, $weekDay, $monthly);
        }
        return $occurrences;
    }

    /**
     * @param $yearDayException
     * @param $monthly
     * @param $orderDay
     * @param $weekDay
     * @param $yearDay
     * @param $calendarDetail
     * @return \Zeedhi\DevOps\Model\Projectmanagement\Entities\CalendarDetail
     */
    protected function setRecurrenceYearly($yearDayException, $monthly, $orderDay, $weekDay, $yearDay, $calendarDetail)
    {
        if (self::EXCEPTION_TRUE == $yearDayException) {
            $calendarDetail = $this->calendarDetailFactory->setRecurrenceYearlyDay($calendarDetail, $yearDay[0],
                $yearDay[1]);
            return $calendarDetail;
        } else {
            $calendarDetail = $this->calendarDetailFactory->setRecurrenceYearlyWeek($calendarDetail, $orderDay,
                $weekDay, $monthly);
            return $calendarDetail;
        }
    }

    /**
     * @param $recurrenceType
     * @param \DateTime $beginDate
     * @param $occurrences
     * @param $exceptionQuantitative
     * @param $endDate
     * @param $every
     * @param $sun
     * @param $mon
     * @param $tue
     * @param $wed
     * @param $thu
     * @param $fri
     * @param $sat
     * @param $yearDayException
     * @param $monthlyDay
     * @param $monthly
     * @param $orderDay
     * @param $weekDay
     * @param $yearDay
     * @return array
     * @throws \Exception
     */
    public function setEndDateOrOccurrences($recurrenceType, \DateTime $beginDate, $occurrences,
                                               $exceptionQuantitative, $endDate, $every, $sun, $mon, $tue, $wed, $thu,
                                               $fri, $sat, $yearDayException, $monthlyDay, $monthly, $orderDay, $weekDay,
                                               $yearDay)
    {
        if ($exceptionQuantitative == self::EXCEPTION_TRUE) {
            $return = $this->setOccurrencesByRecurrence($recurrenceType, $beginDate, $occurrences, $endDate, $every,
                $sun, $mon, $tue, $wed, $thu, $fri, $sat, $yearDayException, $monthlyDay, $monthly, $orderDay, $weekDay,
                $yearDay);
        } else {
            $return = $this->setEndDateByRecurrence($recurrenceType, $beginDate, $occurrences, $endDate, $every, $sun,
                $mon, $tue, $wed, $thu, $fri, $sat, $yearDayException, $monthlyDay, $monthly, $orderDay, $weekDay,
                $yearDay);
        }
        return $return;
    }

    /**
     * @param $recurrenceType
     * @param \DateTime $beginDate
     * @param $occurrences
     * @param $endDate
     * @param $every
     * @param $sun
     * @param $mon
     * @param $tue
     * @param $wed
     * @param $thu
     * @param $fri
     * @param $sat
     * @param $yearDayException
     * @param $monthlyDay
     * @param $monthly
     * @param $orderDay
     * @param $weekDay
     * @param $yearDay
     * @return array
     * @throws \Exception
     */
    protected function setOccurrencesByRecurrence($recurrenceType, \DateTime $beginDate, $occurrences, $endDate, $every,
                                                  $sun, $mon, $tue, $wed, $thu, $fri, $sat, $yearDayException,
                                                  $monthlyDay, $monthly, $orderDay, $weekDay, $yearDay)
    {
        $endDate = clone $beginDate;
        if (self::RECURRENCE_TYPE_DAILY == $recurrenceType) {
            $days = ($occurrences - 1) * $every;
            $endDate->modify("+$days day");
            $return = array($endDate, $occurrences);
        } else if (self::RECURRENCE_TYPE_WEEKLY == $recurrenceType) {
            $endDate = $this->getEndDateForWeekRecurrence($beginDate, $occurrences, $every, $sun, $mon, $tue,
                $wed, $thu, $fri, $sat);
            $return = array($endDate, $occurrences);
        } else if (self::RECURRENCE_TYPE_MONTHLY == $recurrenceType) {
            $endDate = $this->getEndDateForMonthlyRecurrence($beginDate, $occurrences, $every,
                $yearDayException, $monthlyDay, $orderDay, $weekDay);
            $return = array($endDate, $occurrences);
        } else if (self::RECURRENCE_TYPE_YEARLY == $recurrenceType) {
            $endDate = $this->getEndDateForYearlyRecurrence($beginDate, $occurrences, $every,
                $yearDayException, $monthly, $orderDay, $weekDay, $yearDay);
            $return = array($endDate, $occurrences);
        } else {
            throw new \Exception('Tipo de recorrencia não encontrada');
        }
        return $return;
    }

    /**
     * @param $recurrenceType
     * @param \DateTime $beginDate
     * @param $occurrences
     * @param $endDate
     * @param $every
     * @param $sun
     * @param $mon
     * @param $tue
     * @param $wed
     * @param $thu
     * @param $fri
     * @param $sat
     * @param $yearDayException
     * @param $monthlyDay
     * @param $monthly
     * @param $orderDay
     * @param $weekDay
     * @param $yearDay
     * @return array
     * @throws \Exception
     */
    protected function setEndDateByRecurrence($recurrenceType, \DateTime $beginDate, $occurrences, $endDate, $every,
                                              $sun, $mon, $tue, $wed, $thu, $fri, $sat, $yearDayException, $monthlyDay,
                                              $monthly, $orderDay, $weekDay, $yearDay)
    {
        if (self::RECURRENCE_TYPE_DAILY == $recurrenceType) {
            $days = intval($beginDate->diff($endDate)->format("%a"));
            $occurrences = round(($days + 1) / $every);
            $return = array($endDate, $occurrences);
        } else if (self::RECURRENCE_TYPE_WEEKLY == $recurrenceType) {
            $occurrences = $this->getOccurrencesForWeekRecurrence($beginDate, $endDate, $every, $sun, $mon, $tue,
                $wed, $thu, $fri, $sat);
            $return = array($endDate, $occurrences);
        } else if (self::RECURRENCE_TYPE_MONTHLY == $recurrenceType) {
            $occurrences = $this->getOccurrencesForMonthlyRecurrence($beginDate, $endDate, $every,
                $yearDayException, $monthlyDay, $orderDay, $weekDay);
            $return = array($endDate, $occurrences);
        } else if (self::RECURRENCE_TYPE_YEARLY == $recurrenceType) {
            $occurrences = $this->getOccurrencesForYearlyRecurrence($beginDate, $endDate, $every,
                $yearDayException, $monthly, $orderDay, $weekDay, $yearDay);
            $return = array($endDate, $occurrences);
        } else {
            throw new \Exception('Tipo de recorrencia não encontrada');
        }
        return $return;
    }

    /**
     * @param $type
     * @param $workTime
     * @param $calendarDetail
     * @return array
     * @throws \Exception
     */
    protected function setWorkTimeByType($type, $workTime, $calendarDetail)
    {
        if (self::WORK_TYPE == $type) {
            $calendarDetail = $this->calendarDetailFactory->setWorkTime($calendarDetail, $type,
                Utilities::timeToSeconds($workTime));
            $return = array($calendarDetail, $workTime);
        } else if (self::OUT_DAY_TYPE == $type) {
            $calendarDetail = $this->calendarDetailFactory->setWorkTime($calendarDetail, $type, null);
            $workTime = 0;
            $return = array($calendarDetail, $workTime);
        } else {
            throw new \Exception('Tipo de exceção não encontrada');
        }
        return $return;
    }

    /**
     * @param $recurrenceType
     * @param $status
     * @param $every
     * @param $sun
     * @param $mon
     * @param $tue
     * @param $wed
     * @param $thu
     * @param $fri
     * @param $sat
     * @param $yearDayException
     * @param $monthlyDay
     * @param $monthly
     * @param $orderDay
     * @param $weekDay
     * @param $yearDay
     * @param $workTime
     * @param $calendarDetail
     * @return \Zeedhi\DevOps\Model\Projectmanagement\Entities\CalendarDetail
     * @throws \Exception
     */
    protected function setCalendarDetailByRecurrence($recurrenceType, $status, $every, $sun, $mon, $tue, $wed, $thu,
                                                     $fri, $sat, $yearDayException, $monthlyDay, $monthly, $orderDay,
                                                     $weekDay, $yearDay, $workTime, $calendarDetail)
    {
        $this->removeAllWeeklyDays($calendarDetail);
        $this->calendarDetailFactory->clearFieldsRecurrence($calendarDetail);
        if (self::RECURRENCE_TYPE_DAILY == $recurrenceType) {
            $calendarDetail = $this->calendarDetailFactory->setRecurrenceDaily($calendarDetail, $every);
        } else if (self::RECURRENCE_TYPE_WEEKLY == $recurrenceType) {
            $calendarDetail = $this->setRecurrenceWeekly($status, $every, $sun, $mon, $tue, $wed, $thu, $fri, $sat,
                $workTime, $calendarDetail);
        } else if (self::RECURRENCE_TYPE_MONTHLY == $recurrenceType) {
            $calendarDetail = $this->setRecurrenceMonthly($every, $yearDayException, $monthlyDay, $orderDay, $weekDay,
                $calendarDetail);
        } else if (self::RECURRENCE_TYPE_YEARLY == $recurrenceType) {
            $calendarDetail = $this->setRecurrenceYearly($yearDayException, $monthly, $orderDay, $weekDay, $yearDay,
                $calendarDetail);
        } else {
            throw new \Exception('Tipo de recorrencia não encontrada');
        }
        return $calendarDetail;
    }
}